@@ -17,7 +17,8 @@ from decimal import Decimal, DecimalException
 from itertools import takewhile
 from abc import ABCMeta
 from collections.abc import MutableSequence
-
+from typing import cast, ClassVar, Callable, Dict, List, Optional, Union, Type
+from typing import Pattern, Match
 
 #
 # Simple top down parser based on Vaughan Pratt's algorithm (Top Down Operator Precedence).
@@ -38,10 +39,11 @@ from collections.abc import MutableSequence
 #
 
 # Parser special symbols set, that includes the special symbols of TDOP plus two
-# additional special symbols for managing invalid literals and unknown symbols.
+# additional special symbols for managing invalid literals and unknown symbols
+# and source start.
 SPECIAL_SYMBOLS = frozenset((
-    '(string)', '(float)', '(decimal)', '(integer)',
-    '(name)', '(end)', '(invalid)', '(unknown)'
+    '(start)', '(end)', '(string)', '(float)', '(decimal)',
+    '(integer)', '(name)', '(invalid)', '(unknown)',
 ))
 
 SPACE_PATTERN = re.compile(r'\s')
@@ -51,11 +53,11 @@ class ParseError(SyntaxError):
     """An error when parsing source with TDOP parser."""
 
 
-def count_leading_spaces(s):
+def count_leading_spaces(s: str) -> int:
     return sum(1 for _ in takewhile(str.isspace, s))
 
 
-def symbol_to_classname(symbol):
+def symbol_to_classname(symbol: str) -> str:
     """
     Converts a symbol string to an identifier (only alphanumeric and '_').
     """
@@ -81,7 +83,7 @@ def symbol_to_classname(symbol):
     return value.replace(' ', '').replace('-', '').replace('_', '')
 
 
-class MultiLabel(object):
+class MultiLabel:
     """
     Helper class for defining multi-value label for tokens. Useful when a symbol has more roles.
     A label of this type has equivalence with each of its values.
@@ -92,7 +94,7 @@ class MultiLabel(object):
         label == 'function'  # True
         label == 'operator'  # True
     """
-    def __init__(self, *values):
+    def __init__(self, *values: str):
         self.values = values
 
     def __eq__(self, other):
@@ -110,13 +112,13 @@ class MultiLabel(object):
     def __hash__(self):
         return hash(self.values)
 
-    def __contains__(self, item):
+    def __contains__(self, item: str):
         return any(item in v for v in self.values)
 
-    def startswith(self, s):
+    def startswith(self, s: str):
         return any(v.startswith(s) for v in self.values)
 
-    def endswith(self, s):
+    def endswith(self, s: str):
         return any(v.endswith(s) for v in self.values)
 
 
@@ -154,11 +156,11 @@ class Token(MutableSequence):
     covers multiple roles (eg. as XPath function or axis). In those cases the definitive \
     role is defined at parse time (nud and/or led methods) after the token instance creation.
     """
-    symbol = None     # the token identifier
-    lbp = 0           # left binding power
-    rbp = 0           # right binding power
-    pattern = None    # a custom regex pattern for building the tokenizer
-    label = 'symbol'  # optional label
+    lbp = 0  # left binding power
+    rbp = 0  # right binding power
+    symbol: Optional[str] = None   # the token identifier
+    pattern: Optional[str] = None  # a custom regex pattern for building the tokenizer
+    label = 'symbol'               # optional label
 
     __slots__ = '_items', 'parser', 'value', '_source', 'span'
 
@@ -198,9 +200,9 @@ class Token(MutableSequence):
     def __repr__(self):
         symbol, value = self.symbol, self.value
         if value != symbol:
-            return u'%s(value=%r)' % (self.__class__.__name__, value)
+            return '%s(value=%r)' % (self.__class__.__name__, value)
         else:
-            return u'%s()' % self.__class__.__name__
+            return '%s()' % self.__class__.__name__
 
     def __eq__(self, other):
         try:
@@ -310,7 +312,7 @@ class Token(MutableSequence):
             return ParseError('unexpected name %r' % self.value)
         elif self.symbol != '(end)':
             return ParseError('unexpected literal %r' % self.value)
-        elif self.parser.token is None:
+        elif self.parser.token.symbol == '(start)':
             return ParseError('source is empty')
         else:
             return ParseError('unexpected end of source')
@@ -337,7 +339,7 @@ class ParserMeta(type):
             cls.token_base_class = Token
         if not hasattr(cls, 'literals_pattern'):
             cls.literals_pattern = re.compile(
-                r"""'[^']*'|"[^"]*"|(?:\d+|\.\d+)(?:\.\d*)?(?:[Ee][+-]?\d+)?"""
+                    r"""'[^']*'|"[^"]*"|(?:\d+|\.\d+)(?:\.\d*)?(?:[Ee][+-]?\d+)?"""
             )
         if not hasattr(cls, 'name_pattern'):
             cls.name_pattern = re.compile(r'[A-Za-z0-9_]+')
@@ -373,26 +375,32 @@ class Parser(metaclass=ParserMeta):
     """
     SYMBOLS = SPECIAL_SYMBOLS
     token_base_class = Token
-    tokenizer = None
-    symbol_table = {}
+    tokenizer: Optional[Pattern] = None
+    symbol_table: ClassVar[Dict[str, Type[Token]]] = {}
 
-    __slots__ = 'source', 'tokens', 'match', 'token', 'next_token'
+    _start_token: Token
+    token: Token
+    next_token: Token
+    literals_pattern: Pattern
+    name_pattern: Pattern
+
+    __slots__ = 'source', 'tokens', 'match', '_start_token', 'token', 'next_token'
 
     def __init__(self):
         if self.tokenizer is None:
             self.build()
         self.source = ''
         self.tokens = iter(())
-        self.match = None
-        self.token = None
-        self.next_token = None
+        self.match: Optional[Match] = None
+        self._start_token = self.symbol_table['(start)'](self)
+        self.token = self.next_token = self._start_token
 
     def __eq__(self, other):
         return self.token_base_class is other.token_base_class and \
             self.SYMBOLS == other.SYMBOLS and \
             self.symbol_table == other.symbol_table
 
-    def parse(self, source):
+    def parse(self, source: str) -> Token:
         """
         Parses a source code of the formal language. This is the main method that has to be
         called for a parser's instance.
@@ -402,7 +410,7 @@ class Parser(metaclass=ParserMeta):
         """
         try:
             try:
-                self.tokens = iter(self.tokenizer.finditer(source))
+                self.tokens = iter(self.tokenizer.finditer(source))  # type: ignore[union-attr]
             except TypeError as err:
                 token = self.symbol_table['(invalid)'](self, type(source))
                 raise token.wrong_syntax('invalid source type, {}'.format(err))
@@ -415,10 +423,9 @@ class Parser(metaclass=ParserMeta):
         finally:
             self.tokens = iter(())
             self.match = None
-            self.token = None
-            self.next_token = None
+            self.token = self.next_token = self._start_token
 
-    def advance(self, *symbols):
+    def advance(self, *symbols: str) -> Token:
         """
         The Pratt's function for advancing to next token.
 
@@ -427,15 +434,14 @@ class Parser(metaclass=ParserMeta):
         parse error.
         :return: The next token instance.
         """
-        if self.next_token is not None:
-            if self.next_token.symbol == '(end)' or \
-                    symbols and self.next_token.symbol not in symbols:
-                raise self.next_token.wrong_syntax()
+        if self.next_token.symbol == '(end)' or \
+                symbols and self.next_token.symbol not in symbols:
+            raise self.next_token.wrong_syntax()
 
         self.token = self.next_token
         while True:
             try:
-                self.match = next(self.tokens)
+                self.match = cast(Match, next(self.tokens))
             except StopIteration:
                 self.next_token = self.symbol_table['(end)'](self)
                 break
@@ -484,7 +490,7 @@ class Parser(metaclass=ParserMeta):
                     raise RuntimeError(msg % self.match.group())
         return self.next_token
 
-    def advance_until(self, *stop_symbols):
+    def advance_until(self, *stop_symbols: str) -> str:
         """
         Advances until one of the symbols is found or the end of source is reached,
         returning the raw source string placed before. Useful for raw parsing of
@@ -500,10 +506,10 @@ class Parser(metaclass=ParserMeta):
             raise self.next_token.wrong_syntax()
 
         self.token = self.next_token
-        source_chunk = []
+        source_chunk: List[str] = []
         while True:
             try:
-                self.match = next(self.tokens)
+                self.match = cast(Match, next(self.tokens))
             except StopIteration:
                 self.next_token = self.symbol_table['(end)'](self)
                 break
@@ -524,7 +530,7 @@ class Parser(metaclass=ParserMeta):
                     source_chunk.append(self.match.group())
         return ''.join(source_chunk)
 
-    def expression(self, rbp=0):
+    def expression(self, rbp: int = 0) -> Token:
         """
         Pratt's function for parsing an expression. It calls token.nud() and then advances
         until the right binding power is less the left binding power of the next
@@ -545,8 +551,6 @@ class Parser(metaclass=ParserMeta):
     @property
     def position(self):
         """Property that returns the current line and column indexes."""
-        if self.token is None:
-            return 1, 1 + count_leading_spaces(self.source)
         return self.token.position
 
     def is_source_start(self):
@@ -554,8 +558,6 @@ class Parser(metaclass=ParserMeta):
         Returns `True` if the parser is positioned at the start
         of the source, ignoring the spaces.
         """
-        if self.token is None:
-            return True
         return not bool(self.source[0:self.token.span[0]].strip())
 
     def is_line_start(self):
@@ -563,8 +565,6 @@ class Parser(metaclass=ParserMeta):
         Returns `True` if the parser is positioned at the start
         of a source line, ignoring the spaces.
         """
-        if self.token is None:
-            return True
         token_index = self.token.span[0]
         try:
             line_start = self.source[:token_index].rindex('\n') + 1
@@ -583,9 +583,6 @@ class Parser(metaclass=ParserMeta):
         :param after: if `True` considers also the extra spaces after \
         the current token symbol.
         """
-        if self.token is None:
-            return False
-
         start, end = self.token.span
         try:
             if before and start > 0 and self.source[start - 1] in ' \t\n':
@@ -599,7 +596,7 @@ class Parser(metaclass=ParserMeta):
         return string_literal[1:-1].replace("\\'", "'").replace('\\"', '"')
 
     @classmethod
-    def register(cls, symbol, **kwargs):
+    def register(cls, symbol, **kwargs) -> Type[Token]:
         """
         Register/update a token class in the symbol table.
 
@@ -624,7 +621,8 @@ class Parser(metaclass=ParserMeta):
             # Register a new symbol and create a new custom class. The new class
             # name is registered at parser class's module level.
             if symbol not in cls.SYMBOLS:
-                raise NameError('%r is not a symbol of the parser %r.' % (symbol, cls))
+                if symbol != '(start)':  # for backward compatibility
+                    raise NameError('%r is not a symbol of the parser %r.' % (symbol, cls))
 
             kwargs['symbol'] = symbol
             label = kwargs.get('label', 'symbol')
@@ -640,7 +638,7 @@ class Parser(metaclass=ParserMeta):
                 '__qualname__': token_class_name,
                 '__return__': None
             })
-            token_class = ABCMeta(token_class_name, token_class_bases, kwargs)
+            token_class = cast(Type[Token], ABCMeta(token_class_name, token_class_bases, kwargs))
             cls.symbol_table[symbol] = token_class
             MutableSequence.register(token_class)
             setattr(sys.modules[cls.__module__], token_class_name, token_class)
@@ -657,12 +655,12 @@ class Parser(metaclass=ParserMeta):
         return token_class
 
     @classmethod
-    def unregister(cls, symbol):
+    def unregister(cls, symbol: str):
         """Unregister a token class from the symbol table."""
         del cls.symbol_table[symbol.strip()]
 
     @classmethod
-    def duplicate(cls, symbol, new_symbol, **kwargs):
+    def duplicate(cls, symbol: str, new_symbol: str, **kwargs) -> Type[Token]:
         """Duplicate a token class with a new symbol."""
         token_class = cls.symbol_table[symbol]
         new_token_class = cls.register(new_symbol, **kwargs)
@@ -673,7 +671,7 @@ class Parser(metaclass=ParserMeta):
         return new_token_class
 
     @classmethod
-    def literal(cls, symbol, bp=0):
+    def literal(cls, symbol: str, bp: int = 0):
         """Register a token for a symbol that represents a *literal*."""
         def nud(self):
             return self
@@ -684,14 +682,14 @@ class Parser(metaclass=ParserMeta):
         return cls.register(symbol, label='literal', lbp=bp, evaluate=evaluate, nud=nud)
 
     @classmethod
-    def nullary(cls, symbol, bp=0):
+    def nullary(cls, symbol: str, bp: int = 0):
         """Register a token for a symbol that represents a *nullary* operator."""
         def nud(self):
             return self
         return cls.register(symbol, label='operator', lbp=bp, nud=nud)
 
     @classmethod
-    def prefix(cls, symbol, bp=0):
+    def prefix(cls, symbol: str, bp: int = 0):
         """Register a token for a symbol that represents a *prefix* unary operator."""
         def nud(self):
             self[:] = self.parser.expression(rbp=bp),
@@ -699,7 +697,7 @@ class Parser(metaclass=ParserMeta):
         return cls.register(symbol, label='prefix operator', lbp=bp, rbp=bp, nud=nud)
 
     @classmethod
-    def postfix(cls, symbol, bp=0):
+    def postfix(cls, symbol: str, bp: int = 0):
         """Register a token for a symbol that represents a *postfix* unary operator."""
         def led(self, left):
             self[:] = left,
@@ -707,7 +705,7 @@ class Parser(metaclass=ParserMeta):
         return cls.register(symbol, label='postfix operator', lbp=bp, rbp=bp, led=led)
 
     @classmethod
-    def infix(cls, symbol, bp=0):
+    def infix(cls, symbol: str, bp: int = 0):
         """Register a token for a symbol that represents an *infix* binary operator."""
         def led(self, left):
             self[:] = left, self.parser.expression(rbp=bp)
@@ -715,7 +713,7 @@ class Parser(metaclass=ParserMeta):
         return cls.register(symbol, label='operator', lbp=bp, rbp=bp, led=led)
 
     @classmethod
-    def infixr(cls, symbol, bp=0):
+    def infixr(cls, symbol: str, bp: int = 0):
         """Register a token for a symbol that represents an *infixr* binary operator."""
         def led(self, left):
             self[:] = left, self.parser.expression(rbp=bp - 1)
@@ -723,7 +721,7 @@ class Parser(metaclass=ParserMeta):
         return cls.register(symbol, label='operator', lbp=bp, rbp=bp - 1, led=led)
 
     @classmethod
-    def method(cls, symbol, bp=0):
+    def method(cls, symbol: Union[str, Type[Token]], bp: int = 0) -> Callable:
         """
         Register a token for a symbol that represents a custom operator or redefine
         a method for an existing token.
@@ -731,9 +729,10 @@ class Parser(metaclass=ParserMeta):
         token_class = cls.register(symbol, label='operator', lbp=bp, rbp=bp)
 
         def bind(func):
-            assert callable(getattr(token_class, func.__name__, None)), \
-                "The name %r does not match with a callable of %r." % (func.__name__, token_class)
-            setattr(token_class, func.__name__, func)
+            method_name = func.__name__.partition('_')[0]
+            if not callable(getattr(token_class, method_name)):
+                raise TypeError(f"The attribute {method_name!r} is not a callable of {token_class}")
+            setattr(token_class, method_name, func)
             return func
         return bind
 
@@ -743,6 +742,10 @@ class Parser(metaclass=ParserMeta):
         Builds the parser class. Checks if all declared symbols are defined
         and builds a the regex tokenizer using the symbol related patterns.
         """
+        # For backward compatibility with external defined parsers
+        if '(start)' not in cls.symbol_table:
+            cls.register('(start)')
+
         if not cls.SYMBOLS.issubset(cls.symbol_table.keys()):
             unregistered = [s for s in cls.SYMBOLS if s not in cls.symbol_table]
             raise ValueError("The parser %r has unregistered symbols: %r" % (cls, unregistered))
@@ -751,7 +754,7 @@ class Parser(metaclass=ParserMeta):
     build_tokenizer = build  # For backward compatibility
 
     @classmethod
-    def create_tokenizer(cls, symbol_table):
+    def create_tokenizer(cls, symbol_table: Dict[str, Type[Token]]) -> Pattern:
         """
         Returns a regex based tokenizer built from a symbol table of token classes.
         The returned tokenizer skips extra spaces between symbols.
@@ -779,7 +782,7 @@ class Parser(metaclass=ParserMeta):
             else:
                 string_patterns.append(re.escape(symbol))
 
-        symbols_patterns = []
+        symbols_patterns: List[str] = []
         if string_patterns:
             symbols_patterns.append('|'.join(sorted(string_patterns, key=lambda x: -len(x))))
         if character_patterns:
