@@ -12,6 +12,7 @@ import math
 import decimal
 import operator
 from copy import copy
+from typing import cast, Dict, Optional, Tuple
 
 from ..helpers import EQNAME_PATTERN, normalize_sequence_type
 from ..exceptions import MissingContextError, ElementPathKeyError, \
@@ -25,7 +26,7 @@ from ..namespaces import XML_NAMESPACE, XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE
     XPATH_MATH_FUNCTIONS_NAMESPACE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, \
     XSD_UNTYPED_ATOMIC, get_namespace, get_expanded_name, split_expanded_name
 from ..schema_proxy import AbstractSchemaProxy
-from ..xpath_token import XPathToken, XPathFunction
+from ..xpath_token import NargsType, XPathToken, XPathAxis, XPathFunction
 from ..xpath_nodes import XPathNode, TypedElement, AttributeNode, TypedAttribute, \
     is_xpath_node, match_element_node, is_schema_node, is_document_node, \
     match_attribute_node, is_element_node, node_kind
@@ -98,17 +99,22 @@ class XPath1Parser(Parser):
     """
 
     # Labels and symbols admitted after a path step
-    PATH_STEP_LABELS = ('axis', 'kind test')
+    PATH_STEP_LABELS: Tuple[str, ...] = ('axis', 'kind test')
     PATH_STEP_SYMBOLS = {
         '(integer)', '(string)', '(float)', '(decimal)', '(name)', '*', '@', '..', '.', '{'
     }
 
     # Class attributes for compatibility with XPath 2.0+
+    schema: Optional[AbstractSchemaProxy]
+    variable_types: Optional[Dict[str, str]]
+    token: XPathToken
+    next_token: XPathToken
+
     schema = None           # XPath 1.0 doesn't have schema bindings
     variable_types = None   # XPath 1.0 doesn't have in-scope variable types
     xsd_version = '1.0'     # Use XSD 1.0 datatypes for default
     function_namespace = XPATH_FUNCTIONS_NAMESPACE
-    function_signatures = {}
+    function_signatures: Dict[Tuple[QName, int], str] = {}
 
     # https://www.w3.org/TR/xpath-3/#id-reserved-fn-names
     RESERVED_FUNCTION_NAMES = {
@@ -117,7 +123,7 @@ class XPath1Parser(Parser):
         'schema-attribute', 'schema-element', 'switch', 'text', 'typeswitch',
     }
 
-    def __init__(self, namespaces=None, strict=True, *args, **kwargs):
+    def __init__(self, namespaces: Optional[Dict[str, str]] = None, strict=True, *args, **kwargs):
         super(XPath1Parser, self).__init__()
         self.namespaces = self.DEFAULT_NAMESPACES.copy()
         if namespaces is not None:
@@ -125,19 +131,19 @@ class XPath1Parser(Parser):
         self.strict = strict
 
     @property
-    def compatibility_mode(self):
+    def compatibility_mode(self) -> bool:
         """XPath 1.0 compatibility mode."""
         return True
 
     @property
-    def default_namespace(self):
+    def default_namespace(self) -> Optional[str]:
         """
         The default namespace. For XPath 1.0 this value is always `None` because the default
         namespace is ignored (see https://www.w3.org/TR/1999/REC-xpath-19991116/#node-tests).
         """
-        return
+        return None
 
-    def xsd_qname(self, local_name):
+    def xsd_qname(self, local_name: str) -> str:
         """Returns a prefixed QName string for XSD namespace."""
         if self.namespaces.get('xs') == XSD_NAMESPACE:
             return 'xs:%s' % local_name
@@ -149,38 +155,27 @@ class XPath1Parser(Parser):
         raise xpath_error('XPST0081', 'Missing XSD namespace registration')
 
     @staticmethod
-    def unescape(string_literal):
+    def unescape(string_literal: str) -> str:
         if string_literal.startswith("'"):
             return string_literal[1:-1].replace("''", "'")
         else:
             return string_literal[1:-1].replace('""', '"')
 
     @classmethod
-    def axis(cls, symbol, reverse_axis=False, bp=80):
+    def axis(cls, symbol: str, reverse_axis: bool = False, bp: int = 80):
         """Register a token for a symbol that represents an XPath *axis*."""
-        def nud_(self):
-            self.parser.advance('::')
-            self.parser.expected_name(
-                '(name)', '*', 'text', 'node', 'document-node',
-                'comment', 'processing-instruction', 'attribute',
-                'schema-attribute', 'element', 'schema-element'
-            )
-            self[:] = self.parser.expression(rbp=bp),
-            return self
-
-        pattern = r'\b%s(?=\s*\:\:|\s*\(\:.*\:\)\s*\:\:)' % cls.name_pattern.pattern
-        return cls.register(
-            symbol, pattern=pattern, label='axis',
-            reverse_axis=reverse_axis, lbp=bp, rbp=bp, nud=nud_
-        )
+        return cls.register(symbol, label='axis', bases=(XPathAxis,),
+                            reverse_axis=reverse_axis, lbp=bp, rbp=bp)
 
     @classmethod
-    def function(cls, symbol, nargs=None, sequence_types=(), label='function', bp=90):
+    def function(cls, symbol: str, nargs: NargsType = None,
+                 sequence_types: Tuple[str, ...] = (),
+                 label: str = 'function', bp: int = 90):
         """
         Registers a token class for a symbol that represents an XPath function.
         """
         if 'function' not in label:
-            pass
+            pass  # kind test or sequence type
         elif symbol in cls.RESERVED_FUNCTION_NAMES:
             raise ElementPathValueError(f'{symbol!r} is a reserved function name')
         elif sequence_types:
@@ -212,8 +207,8 @@ class XPath1Parser(Parser):
         return cls.register(symbol, nargs=nargs, sequence_types=sequence_types, label=label,
                             bases=(XPathFunction,), lbp=bp, rbp=bp)
 
-    def parse(self, source):
-        root_token = super(XPath1Parser, self).parse(source)
+    def parse(self, source: str):
+        root_token = cast(XPathToken, super(XPath1Parser, self).parse(source))
         try:
             root_token.evaluate()  # Static context evaluation
         except MissingContextError:
@@ -430,6 +425,16 @@ method = XPath1Parser.method
 function = XPath1Parser.function
 axis = XPath1Parser.axis
 
+###
+# Special symbols
+register('(start)')
+register('(end)')
+literal('(string)')
+literal('(float)')
+literal('(decimal)')
+literal('(integer)')
+literal('(invalid)')
+literal('(unknown)')
 
 ###
 # Simple symbols
@@ -440,18 +445,8 @@ register('::')
 register('}')
 
 
-###
-# Literals
-literal('(string)')
-literal('(float)')
-literal('(decimal)')
-literal('(integer)')
-literal('(invalid)')
-literal('(unknown)')
-
-
 @method(register('(name)', bp=10, label='literal'))
-def nud(self):
+def nud_name_literal(self):
     if self.parser.next_token.symbol == '(':
         if self.parser.version >= '3.0':
             pass
@@ -472,12 +467,12 @@ def nud(self):
 
 
 @method('(name)')
-def evaluate(self, context=None):
+def evaluate_name_literal(self, context=None):
     return [x for x in self.select(context)]
 
 
 @method('(name)')
-def select(self, context=None):
+def select_name_literal(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -534,7 +529,7 @@ def select(self, context=None):
 ###
 # Namespace prefix reference
 @method(':', bp=95)
-def led(self, left):
+def led_namespace_prefix(self, left):
     if self.parser.version == '1.0':
         left.expected('(name)')
     else:
@@ -568,14 +563,14 @@ def led(self, left):
 
 
 @method(':')
-def evaluate(self, context=None):
+def evaluate_namespace_prefix(self, context=None):
     if self[1].label.endswith('function'):
         return self[1].evaluate(context)
     return [x for x in self.select(context)]
 
 
 @method(':')
-def select(self, context=None):
+def select_namespace_prefix(self, context=None):
     if self[1].label.endswith('function'):
         value = self[1].evaluate(context)
         if isinstance(value, list):
@@ -629,7 +624,7 @@ def select(self, context=None):
 ###
 # Namespace URI as in ElementPath
 @method('{', bp=95)
-def nud(self):
+def nud_namespace_uri(self):
     if self.parser.strict and self.symbol == '{':
         raise self.wrong_syntax("not allowed symbol if parser has strict=True")
 
@@ -650,14 +645,14 @@ def nud(self):
 
 
 @method('{')
-def evaluate(self, context=None):
+def evaluate_namespace_uri(self, context=None):
     if self[1].label.endswith('function'):
         return self[1].evaluate(context)
     return [x for x in self.select(context)]
 
 
 @method('{')
-def select(self, context=None):
+def select_namespace_uri(self, context=None):
     if self[1].label.endswith('function'):
         yield self[1].evaluate(context)
         return
@@ -687,7 +682,7 @@ def select(self, context=None):
 ###
 # Variables
 @method('$', bp=90)
-def nud(self):
+def nud_variable_reference(self):
     self.parser.expected_name('(name)')
     self[:] = self.parser.expression(rbp=90),
     if ':' in self[0].value:
@@ -696,7 +691,7 @@ def nud(self):
 
 
 @method('$')
-def evaluate(self, context=None):
+def evaluate_variable_reference(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -709,7 +704,7 @@ def evaluate(self, context=None):
 ###
 # Nullary operators (use only the context)
 @method(nullary('*'))
-def select(self, context=None):
+def select_wildcard(self, context=None):
     if self:
         # Product operator
         item = self.evaluate(context)
@@ -749,7 +744,7 @@ def select(self, context=None):
 
 
 @method(nullary('.'))
-def select(self, context=None):
+def select_self_shortcut(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -783,7 +778,7 @@ def select(self, context=None):
 
 
 @method(nullary('..'))
-def select(self, context=None):
+def select_parent_shortcut(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -796,13 +791,13 @@ def select(self, context=None):
 ###
 # Logical Operators
 @method(infix('or', bp=20))
-def evaluate(self, context=None):
+def evaluate_or_operator(self, context=None):
     return self.boolean_value(self[0].evaluate(copy(context))) or \
         self.boolean_value(self[1].evaluate(copy(context)))
 
 
 @method(infix('and', bp=25))
-def evaluate(self, context=None):
+def evaluate_and_operator(self, context=None):
     return self.boolean_value(self[0].evaluate(copy(context))) and \
         self.boolean_value(self[1].evaluate(copy(context)))
 
@@ -815,7 +810,7 @@ def evaluate(self, context=None):
 @method('>', bp=30)
 @method('<=', bp=30)
 @method('>=', bp=30)
-def led(self, left):
+def led_comparison_operators(self, left):
     if left.symbol in OPERATORS_MAP:
         raise self.wrong_syntax()
     self[:] = left, self.parser.expression(rbp=30)
@@ -828,7 +823,7 @@ def led(self, left):
 @method('>')
 @method('<=')
 @method('>=')
-def evaluate(self, context=None):
+def evaluate_comparison_operators(self, context=None):
     op = OPERATORS_MAP[self.symbol]
     try:
         return any(op(x1, x2) for x1, x2 in self.iter_comparison_data(context))
@@ -841,7 +836,7 @@ def evaluate(self, context=None):
 ###
 # Numerical operators
 @method(infix('+', bp=40))
-def evaluate(self, context=None):
+def evaluate_plus_operator(self, context=None):
     if len(self) == 1:
         arg = self.get_argument(context, cls=NumericProxy)
         if arg is not None:
@@ -863,7 +858,7 @@ def evaluate(self, context=None):
 
 
 @method(infix('-', bp=40))
-def evaluate(self, context=None):
+def evaluate_minus_operator(self, context=None):
     if len(self) == 1:
         arg = self.get_argument(context, cls=NumericProxy)
         if arg is not None:
@@ -886,13 +881,13 @@ def evaluate(self, context=None):
 
 @method('+')
 @method('-')
-def nud(self):
+def nud_plus_minus_operators(self):
     self[:] = self.parser.expression(rbp=70),
     return self
 
 
 @method(infix('*', bp=45))
-def evaluate(self, context=None):
+def evaluate_multiply_operator(self, context=None):
     if self:
         op1, op2 = self.get_operands(context, cls=ArithmeticProxy)
         if op1 is not None:
@@ -929,7 +924,7 @@ def evaluate(self, context=None):
 
 
 @method(infix('div', bp=45))
-def evaluate(self, context=None):
+def evaluate_div_operator(self, context=None):
     dividend, divisor = self.get_operands(context, cls=ArithmeticProxy)
     if dividend is None:
         return
@@ -964,7 +959,7 @@ def evaluate(self, context=None):
 
 
 @method(infix('mod', bp=45))
-def evaluate(self, context=None):
+def evaluate_mod_operator(self, context=None):
     op1, op2 = self.get_operands(context, cls=NumericProxy)
     if op1 is not None:
         if op2 == 0 and isinstance(op2, float):
@@ -987,7 +982,7 @@ def evaluate(self, context=None):
 @method('and')
 @method('div')
 @method('mod')
-def nud(self):
+def nud_logical_div_mod_operators(self):
     token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
     return token.nud()
 
@@ -995,7 +990,7 @@ def nud(self):
 ###
 # Union expressions
 @method('|', bp=50)
-def led(self, left):
+def led_union_operator(self, left):
     self.cut_and_sort = True
     if left.symbol in {'|', 'union'}:
         left.cut_and_sort = False
@@ -1004,11 +999,11 @@ def led(self, left):
 
 
 @method('|')
-def select(self, context=None):
+def select_union_operator(self, context=None):
     if context is None:
         raise self.missing_context()
 
-    results = {item for k in range(2) for item in self[k].select(context.copy())}
+    results = {item for k in range(2) for item in self[k].select(copy(context))}
     if any(not is_xpath_node(x) for x in results):
         raise self.error('XPTY0004', 'only XPath nodes are allowed')
     elif not self.cut_and_sort:
@@ -1020,7 +1015,7 @@ def select(self, context=None):
 ###
 # Path expressions
 @method('//', bp=75)
-def nud(self):
+def nud_descendant_path(self):
     if self.parser.next_token.label not in self.parser.PATH_STEP_LABELS:
         self.parser.expected_name(*self.parser.PATH_STEP_SYMBOLS)
 
@@ -1029,7 +1024,7 @@ def nud(self):
 
 
 @method('/', bp=75)
-def nud(self):
+def nud_child_path(self):
     if self.parser.next_token.label not in self.parser.PATH_STEP_LABELS:
         try:
             self.parser.expected_name(*self.parser.PATH_STEP_SYMBOLS)
@@ -1042,7 +1037,7 @@ def nud(self):
 
 @method('//')
 @method('/')
-def led(self, left):
+def led_child_or_descendant_path(self, left):
     if self.parser.next_token.label not in self.parser.PATH_STEP_LABELS:
         self.parser.expected_name(*self.parser.PATH_STEP_SYMBOLS)
 
@@ -1051,7 +1046,7 @@ def led(self, left):
 
 
 @method('/')
-def select(self, context=None):
+def select_child_path(self, context=None):
     """
     Child path expression. Selects child:: axis as default (when bind to '*' or '(name)').
     """
@@ -1092,7 +1087,7 @@ def select(self, context=None):
 
 
 @method('//')
-def select(self, context=None):
+def select_descendant_path(self, context=None):
     # Note: // is short for /descendant-or-self::node()/, so the axis
     #   is left to None. Use descendant:: only if next-step uses child
     #   axis, to preserve document order.
@@ -1119,14 +1114,14 @@ def select(self, context=None):
 ###
 # Predicate filters
 @method('[', bp=80)
-def led(self, left):
+def led_predicate(self, left):
     self[:] = left, self.parser.expression()
     self.parser.advance(']')
     return self
 
 
 @method('[')
-def select(self, context=None):
+def select_predicate(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -1135,7 +1130,7 @@ def select(self, context=None):
                 and not is_xpath_node(context.item):
             raise self.error('XPTY0020')
 
-        predicate = [x for x in self[1].select(context.copy())]
+        predicate = [x for x in self[1].select(copy(context))]
         if len(predicate) == 1 and isinstance(predicate[0], NumericProxy):
             if context.position == predicate[0]:
                 yield context.item
@@ -1146,17 +1141,17 @@ def select(self, context=None):
 ###
 # Parenthesized expressions
 @method('(', bp=100)
-def nud(self):
+def nud_parenthesized_expr(self):
     self[:] = self.parser.expression(),
     self.parser.advance(')')
     return self
 
 
 @method('(')
-def evaluate(self, context=None):
+def evaluate_parenthesized_expr(self, context=None):
     return self[0].evaluate(context)
 
 
 @method('(')
-def select(self, context=None):
+def select_parenthesized_expr(self, context=None):
     return self[0].select(context)
