@@ -18,6 +18,7 @@ from collections.abc import MutableSequence
 from copy import copy
 from decimal import Decimal, DivisionByZero
 from urllib.parse import urlparse
+from typing import cast
 
 from ..helpers import normalize_sequence_type
 from ..exceptions import ElementPathError, ElementPathTypeError, \
@@ -28,7 +29,7 @@ from ..namespaces import XSD_NAMESPACE, XML_NAMESPACE, XLINK_NAMESPACE, \
 from ..datatypes import UntypedAtomic, QName, AnyURI, Duration, Integer
 from ..xpath_nodes import TypedElement, is_xpath_node, \
     match_attribute_node, is_element_node, is_document_node
-from ..xpath_token import UNICODE_CODEPOINT_COLLATION, XPathFunction
+from ..xpath_token import XPathToken, UNICODE_CODEPOINT_COLLATION, XPathFunction
 from ..xpath1 import XPath1Parser
 from ..xpath_context import XPathSchemaContext
 from ..schema_proxy import AbstractSchemaProxy
@@ -337,9 +338,10 @@ class XPath2Parser(XPath1Parser):
                                    nud=nud_, evaluate=evaluate_, cast=cast_)
 
         def bind(func):
-            assert func.__name__ == 'cast', \
-                "The function name must be 'cast', not %r." % func.__name__
-            setattr(token_class, func.__name__, func)
+            method_name = func.__name__.partition('_')[0]
+            if method_name != 'cast':
+                raise ValueError("The function name must be 'cast' or starts with 'cast_'")
+            setattr(token_class, method_name, func)
             return func
         return bind
 
@@ -394,7 +396,7 @@ class XPath2Parser(XPath1Parser):
         return 'symbol_table' in self.__dict__
 
     def parse(self, source):
-        root_token = super(XPath1Parser, self).parse(source)
+        root_token = cast(XPathToken, super(XPath1Parser, self).parse(source))
         if root_token.label == 'sequence type':
             raise root_token.error('XPST0003', "not allowed in XPath expression")
 
@@ -461,21 +463,21 @@ register(':)')
 
 @method('as')
 @method('of')
-def nud(self):
+def nud_as_and_of_symbols(self):
     raise self.error('XPDY0002')  # Dynamic context required
 
 
 ###
 # Variables
 @method('$', bp=90)
-def nud(self):
+def nud_variable_reference(self):
     self.parser.expected_name('(name)')
     self[:] = self.parser.expression(rbp=90),
     return self
 
 
 @method('$')
-def evaluate(self, context=None):
+def evaluate_variable_reference(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -511,7 +513,7 @@ XPath2Parser.duplicate('|', 'union')
 
 @method(infix('intersect', bp=55))
 @method(infix('except', bp=55))
-def select(self, context=None):
+def select_intersect_and_except_operators(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -528,7 +530,7 @@ def select(self, context=None):
 ###
 # 'if' expression
 @method('if', bp=20)
-def nud(self):
+def nud_if_expression(self):
     if self.parser.next_token.symbol != '(':
         token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
         return token.nud()
@@ -544,7 +546,7 @@ def nud(self):
 
 
 @method('if')
-def evaluate(self, context=None):
+def evaluate_if_expression(self, context=None):
     if self.boolean_value(self[0].evaluate(copy(context))):
         return self[1].evaluate(context)
     else:
@@ -552,7 +554,7 @@ def evaluate(self, context=None):
 
 
 @method('if')
-def select(self, context=None):
+def select_if_expression(self, context=None):
     if self.boolean_value([x for x in self[0].select(copy(context))]):
         yield from self[1].select(context)
     else:
@@ -563,7 +565,7 @@ def select(self, context=None):
 # Quantified expressions
 @method('some', bp=20)
 @method('every', bp=20)
-def nud(self):
+def nud_quantified_expressions(self):
     del self[:]
     if self.parser.next_token.symbol != '$':
         token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
@@ -591,7 +593,7 @@ def nud(self):
 
 @method('some')
 @method('every')
-def evaluate(self, context=None):
+def evaluate_quantified_expressions(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -614,7 +616,7 @@ def evaluate(self, context=None):
 ###
 # 'for' expressions
 @method('for', bp=20)
-def nud(self):
+def nud_for_expression(self):
     del self[:]
     if self.parser.next_token.symbol != '$':
         token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
@@ -641,7 +643,7 @@ def nud(self):
 
 
 @method('for')
-def select(self, context=None):
+def select_for_expression(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -658,7 +660,7 @@ def select(self, context=None):
 # Sequence type based
 @method('instance', bp=60)
 @method('treat', bp=61)
-def led(self, left):
+def led_sequence_type_based_expressions(self, left):
     self.parser.advance('of' if self.symbol == 'instance' else 'as')
     if self.parser.next_token.label not in ('kind test', 'sequence type'):
         self.parser.expected_name('(name)', ':')
@@ -677,7 +679,7 @@ def led(self, left):
 
 
 @method('instance')
-def evaluate(self, context=None):
+def evaluate_instance_expression(self, context=None):
     occurs = self[2].symbol if len(self) > 2 else None
     position = None
 
@@ -718,7 +720,7 @@ def evaluate(self, context=None):
 
 
 @method('treat')
-def evaluate(self, context=None):
+def evaluate_treat_expression(self, context=None):
     occurs = self[2].symbol if len(self) > 2 else None
     position = None
     castable_expr = []
@@ -765,7 +767,7 @@ def evaluate(self, context=None):
 # Simple type based
 @method('castable', bp=62)
 @method('cast', bp=63)
-def led(self, left):
+def led_cast_expressions(self, left):
     self.parser.advance('as')
     self.parser.expected_name('(name)', ':')
     self[:] = left, self.parser.expression(rbp=self.rbp)
@@ -777,7 +779,7 @@ def led(self, left):
 
 @method('castable')
 @method('cast')
-def evaluate(self, context=None):
+def evaluate_cast_expressions(self, context=None):
     try:
         atomic_type = get_expanded_name(self[1].source, namespaces=self.parser.namespaces)
     except KeyError as err:
@@ -836,7 +838,7 @@ def evaluate(self, context=None):
 ###
 # Comma operator - concatenate items or sequences
 @method(infix(',', bp=5))
-def evaluate(self, context=None):
+def evaluate_comma_operator(self, context=None):
     results = []
     for op in self:
         result = op.evaluate(context)
@@ -848,15 +850,15 @@ def evaluate(self, context=None):
 
 
 @method(',')
-def select(self, context=None):
+def select_comma_operator(self, context=None):
     for op in self:
         yield from op.select(context=copy(context))
 
 
 ###
-# Parenthesized expressions: XPath 2.0 admits the empty case ().
+# Parenthesized expression: XPath 2.0 admits the empty case ().
 @method('(', bp=100)
-def nud(self):
+def nud_parenthesized_expression(self):
     if self.parser.next_token.symbol != ')':
         self[:] = self.parser.expression(),
     self.parser.advance(')')
@@ -864,12 +866,12 @@ def nud(self):
 
 
 @method('(')
-def evaluate(self, context=None):
+def evaluate_parenthesized_expression(self, context=None):
     return self[0].evaluate(context) if self else []
 
 
 @method('(')
-def select(self, context=None):
+def select_parenthesized_expression(self, context=None):
     return self[0].select(context) if self else iter(())
 
 
@@ -884,7 +886,7 @@ def select(self, context=None):
 @method('gt', bp=30)
 @method('le', bp=30)
 @method('ge', bp=30)
-def led(self, left):
+def led_value_comparison_operators(self, left):
     if left.symbol in {'eq', 'ne', 'lt', 'le', 'gt', 'ge'}:
         raise self.wrong_syntax()
     self[:] = left, self.parser.expression(rbp=30)
@@ -897,7 +899,7 @@ def led(self, left):
 @method('gt')
 @method('le')
 @method('ge')
-def evaluate(self, context=None):
+def evaluate_value_comparison_operators(self, context=None):
     operands = [self[0].get_atomized_operand(context=copy(context)),
                 self[1].get_atomized_operand(context=copy(context))]
 
@@ -935,7 +937,7 @@ def evaluate(self, context=None):
 ###
 # Node comparison
 @method('is', bp=30)
-def led(self, left):
+def led_node_comparison(self, left):
     if left.symbol == 'is':
         raise self.wrong_syntax()
     self[:] = left, self.parser.expression(rbp=30)
@@ -945,7 +947,7 @@ def led(self, left):
 @method('is')
 @method(infix('<<', bp=30))
 @method(infix('>>', bp=30))
-def evaluate(self, context=None):
+def evaluate_node_comparison(self, context=None):
     symbol = self.symbol
 
     left = [x for x in self[0].select(context)]
@@ -977,7 +979,7 @@ def evaluate(self, context=None):
 ###
 # Range expression
 @method('to', bp=35)
-def led(self, left):
+def led_range_expression(self, left):
     if left.symbol == 'to':
         raise self.wrong_syntax()
     self[:] = left, self.parser.expression(rbp=35)
@@ -985,7 +987,7 @@ def led(self, left):
 
 
 @method('to')
-def evaluate(self, context=None):
+def evaluate_range_expression(self, context=None):
     start, stop = self.get_operands(context, cls=Integer)
     try:
         return [x for x in range(start, stop + 1)]
@@ -994,14 +996,14 @@ def evaluate(self, context=None):
 
 
 @method('to')
-def select(self, context=None):
+def select_range_expression(self, context=None):
     yield from self.evaluate(context)
 
 
 ###
 # Numerical operators
 @method(infix('idiv', bp=45))
-def evaluate(self, context=None):
+def evaluate_idiv_operator(self, context=None):
     op1, op2 = self.get_operands(context)
     if op1 is None or op2 is None:
         raise self.error('XPST0005')
@@ -1043,7 +1045,7 @@ def evaluate(self, context=None):
 @method('treat')
 @method('castable')
 @method('cast')
-def nud(self):
+def nud_disambiguation_of_infix_operators(self):
     token = self.parser.symbol_table['(name)'](self.parser, self.symbol)
     return token.nud()
 
@@ -1051,7 +1053,7 @@ def nud(self):
 ###
 # Kind tests (sequence types that can appear also in XPath expressions)
 @method(function('document-node', nargs=(0, 1), label='kind test'))
-def select(self, context=None):
+def select_document_node_kind_test(self, context=None):
     if context is None:
         raise self.missing_context()
     elif not self:
@@ -1067,7 +1069,7 @@ def select(self, context=None):
 
 
 @method('document-node')
-def nud(self):
+def nud_document_node_kind_test(self):
     self.parser.advance('(')
     if self.parser.next_token.symbol in ('element', 'schema-element'):
         self[0:] = self.parser.expression(5),
@@ -1081,7 +1083,7 @@ def nud(self):
 
 
 @method(function('element', nargs=(0, 2), label='kind test'))
-def select(self, context=None):
+def select_element_kind_test(self, context=None):
     if context is None:
         raise self.missing_context()
     elif not self:
@@ -1099,7 +1101,7 @@ def select(self, context=None):
 
 
 @method('element')
-def nud(self):
+def nud_element_kind_test(self):
     self.parser.advance('(')
     if self.parser.next_token.symbol != ')':
         self.parser.expected_name('(name)', ':', '*', message='a QName or a wildcard expected')
@@ -1114,7 +1116,7 @@ def nud(self):
 
 
 @method(function('schema-attribute', nargs=1, label='kind test'))
-def select(self, context=None):
+def select_schema_attribute_kind_test(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -1133,7 +1135,7 @@ def select(self, context=None):
 
 
 @method(function('schema-element', nargs=1, label='kind test'))
-def select(self, context=None):
+def select_schema_element_kind_test(self, context=None):
     if context is None:
         raise self.missing_context()
 
@@ -1154,7 +1156,7 @@ def select(self, context=None):
 
 @method('schema-attribute')
 @method('schema-element')
-def nud(self):
+def nud_schema_node_kind_test(self):
     self.parser.advance('(')
     self.parser.expected_name('(name)', ':', message='a QName expected')
     self[0:] = self.parser.expression(5),
@@ -1177,7 +1179,7 @@ register('attribute', lbp=90, rbp=90, label=('kind test', 'axis'),
 
 
 @method('attribute')
-def nud(self):
+def nud_attribute_kind_test_or_axis(self):
     if self.parser.next_token.symbol == '::':
         self.parser.advance('::')
         self.parser.expected_name(
@@ -1199,7 +1201,7 @@ def nud(self):
 
 
 @method('attribute')
-def select(self, context=None):
+def select_attribute_kind_test_or_axis(self, context=None):
     if context is None:
         raise self.missing_context()
     elif self.label == 'axis':
