@@ -25,11 +25,12 @@ import math
 from copy import copy
 from decimal import Decimal
 from itertools import product
-from typing import Union
+from typing import Optional, Tuple, Union
 import urllib.parse
 from xml.etree.ElementTree import Element
 
 from .exceptions import ElementPathError, ElementPathValueError, XPATH_ERROR_CODES
+from .helpers import ordinal
 from .namespaces import XQT_ERRORS_NAMESPACE, XSD_NAMESPACE, \
     XPATH_FUNCTIONS_NAMESPACE, XPATH_MATH_FUNCTIONS_NAMESPACE, \
     XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XSI_NIL
@@ -45,23 +46,10 @@ from .tdop import Token, MultiLabel
 from .xpath_context import XPathSchemaContext
 
 UNICODE_CODEPOINT_COLLATION = "http://www.w3.org/2005/xpath-functions/collation/codepoint"
-
 XSD_SPECIAL_TYPES = {XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE}
 
-
-def ordinal(n):
-    if n in {11, 12, 13}:
-        return '%dth' % n
-
-    least_significant_digit = n % 10
-    if least_significant_digit == 1:
-        return '%dst' % n
-    elif least_significant_digit == 2:
-        return '%dnd' % n
-    elif least_significant_digit == 3:
-        return '%drd' % n
-    else:
-        return '%dth' % n
+# Type annotations aliases
+NargsType = Optional[Union[int, Tuple[int, Optional[int]]]]
 
 
 class XPathToken(Token):
@@ -1075,6 +1063,22 @@ class XPathToken(Token):
         return self.error('XPST0081', message)
 
 
+class XPathAxis(XPathToken):
+    pattern = r'\b[^\d\W][\w.\-\xb7\u0300-\u036F\u203F\u2040]*(?=\s*\:\:|\s*\(\:.*\:\)\s*\:\:)'
+    label = 'axis'
+    reverse_axis = False
+
+    def nud(self):
+        self.parser.advance('::')
+        self.parser.expected_name(
+            '(name)', '*', 'text', 'node', 'document-node',
+            'comment', 'processing-instruction', 'attribute',
+            'schema-attribute', 'element', 'schema-element'
+        )
+        self[:] = self.parser.expression(rbp=self.rbp),
+        return self
+
+
 class ValueToken(XPathToken):
     """
     A dummy token for encapsulating a value.
@@ -1092,13 +1096,13 @@ class XPathFunction(XPathToken):
     """
     A token for processing XPath functions.
     """
-    _name = None
+    _name: Optional[QName] = None
     pattern = r'\b[^\d\W][\w.\-\xb7\u0300-\u036F\u203F\u2040]*(?=\s*(?:\(\:.*\:\))?\s*\((?!\:))'
 
-    sequence_types = ()
+    sequence_types: Tuple[str, ...] = ()
     "Sequence types of arguments and of the return value of the function."
 
-    nargs = None
+    nargs: NargsType = None
     "Number of arguments: a single value or a couple with None that means unbounded."
 
     def __init__(self, parser, nargs=None):
